#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;

using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text.RegularExpressions;

#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "WakeOnLan", Category = "(Network)", Help = "Wake On Lan by a given MAC adress. The five packets of the address can be separated by either \':\' or \'-\' ", Tags = "remote sebl velcrome")]
	#endregion PluginInfo
	public class NetworkWOLNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("Mac Adress", DefaultString = "00:00:00:00:00:00")]
		ISpread<string> FMac;

		[Input("Broadcast Address", DefaultString = "192.168.255.255")]
		IDiffSpread<string> FAddress;

		[Input("Port", DefaultValue = 9, Visibility = PinVisibility.Hidden )]
		ISpread<int> FPort;
		
		[Input("Wake", IsBang=true)]
		IDiffSpread<bool> FWake;
		
		[Output("Status")]
		ISpread<string> FStatus;
		
		[Import()]
		ILogger FLogger;
		#endregion fields & pins
		
		
		public void Evaluate(int SpreadMax)
		{
			// abort, if Wake pin did not change 
			if (!FWake.IsChanged) return;

			// abort, if no Wake is on 
			bool anyWake = false;
			for (int i = 0; i < SpreadMax; i++)	if (FWake[i]) anyWake=true;
			if (!anyWake) return;
			
			FStatus.SliceCount = SpreadMax;
			for (int i = 0; i < SpreadMax; i++)	{
				if (FWake[i])
				{
					string[] macDigits = null;
					string MAC = null;
					byte[] WOLSignal = new byte[102];
					
					// parse Mac address
					if (FMac[i].Contains("-")) {
						macDigits = FMac[i].Split('-');
					} else if (FMac[i].Contains(":")) {
						macDigits = FMac[i].Split(':');
					} else {
						macDigits =  Split(FMac[i],2).ToArray();
					}
					bool isValid = Regex.IsMatch(string.Join(":", macDigits), "^([0-9A-F]{2}[:-]){5}([0-9A-F]{2})$");
					
					if (isValid)
					{
						FStatus[i] = "Wake UP";
						for (int h = 0; h < 6; h++) {
							WOLSignal[h] = 0xFF; // Magic Packet Header
						}
						int start = 6;
						for (int j = 0; j < 16; j++)
							for (int x = 0; x < 6; x++)
							{
								int index = start + j * 6 + x; 
								WOLSignal[index] = Convert.ToByte(macDigits[x], 16); // Magic Packet Body
							}
						
						// send Magic Packet
						UdpClient WOLclient = new UdpClient();
//						WOLclient.EnableBroadcast = true;
//						WOLclient.Client.SetSocketOption(SocketOptionLevel.Socket,SocketOptionName.Broadcast,0);
						WOLclient.Connect(FAddress[i], FPort[i]);
						WOLclient.Send(WOLSignal, WOLSignal.Length);
					} else FStatus[i] = "incorrect MAC address!";
				} else {
					FStatus[i] = "Skipped";
				}
			}
		}
		
		static IEnumerable<string> Split(string str, int chunkSize)
		{
			return Enumerable.Range(0, str.Length / chunkSize)
			.Select(i => str.Substring(i * chunkSize, chunkSize));
		}
	}
}
